import logging
import types
from typing import Type, Literal, TYPE_CHECKING, Optional

from cloudformation_cli_python_lib.interface import BaseModel
from cloudformation_cli_python_lib.interface import (
    BaseResourceHandlerRequest as _BaseResourceHandlerRequest,
)

from cf_extension_core.resource_base import ResourceBase as _ResourceBase

if TYPE_CHECKING:
    from mypy_boto3_dynamodb.service_resource import DynamoDBServiceResource
else:
    DynamoDBServiceResource = object

# Module Logger
logger = logging.getLogger(__name__)


class ResourceUpdate(_ResourceBase):
    """
    Easily usable class that can be used to Update resources in the custom resource code.
    """

    # #Create Handler use case - as we update via callbacks, continue to update row
    # with dynamodb_update(primary_identifier=self._callback_context["working_model"].ReadOnlyIdentifier,
    #                      request=self._request) as DB:
    #
    #     #Arbitrary Code
    #     DB.update_model(model=self._callback_context["working_model"])
    #
    #
    # #Update Use case
    # with dynamodb_update(primary_identifier=self._request.previousResourceState.ReadOnlyIdentifier,
    #                      request=self._request) as DB:
    #
    #     # Arbitrary Code
    #     DB.update_model(model=self._callback_context["working_model"])

    def __init__(
        self,
        request: _BaseResourceHandlerRequest,
        db_resource: DynamoDBServiceResource,
        primary_identifier: str,
        type_name: str,
    ):

        super().__init__(
            request=request,
            db_resource=db_resource,
            primary_identifier=primary_identifier,
            type_name=type_name,
        )

        self._updated_model: Optional[BaseModel] = None
        self._was_model_updated = False

    def update_model(
        self,
        updated_model: _ResourceBase.T,
    ) -> None:

        if self._primary_identifier is None:
            raise Exception("Primary Identifier cannot be Null")

        if updated_model is None:
            raise Exception("The current_model cannot be Null")

        self._was_model_updated = True
        self._updated_model = updated_model

    def __enter__(self) -> "ResourceUpdate":
        logger.info("DynamoUpdate Enter... ")

        # Check to see if the row/resource is not found
        self._not_found_check()

        logger.info("DynamoUpdate Enter Completed")
        return self

    def __exit__(
        self,
        exception_type: Optional[Type[BaseException]],
        exception_value: Optional[BaseException],
        traceback: Optional[types.TracebackType],
    ) -> Literal[False]:

        logger.info("DynamoUpdate Exit...")

        if exception_type is None:
            logger.info("Has Failure = False")

            if self._was_model_updated:
                logger.info("Row being Updated")
                self._db_item_update_model(model=self._updated_model)
        else:

            # We failed in update logic
            logger.info("Has Failure = True, row NOT updated")

            # Failed during update of resource for any number of reasons
            # Assuming it failed with no resource actually changed from a dynamo perspective.
            # Dont update the Row

        logger.info("DynamoUpdate Exit Completed")

        # let exception flourish always
        return False



import logging
import types
from typing import Type, Literal, TYPE_CHECKING, cast, Optional

import cloudformation_cli_python_lib.exceptions
from cloudformation_cli_python_lib.interface import BaseModel, BaseResourceHandlerRequest

from cf_extension_core.resource_base import ResourceBase as _ResourceBase

if TYPE_CHECKING:
    from mypy_boto3_dynamodb.service_resource import DynamoDBServiceResource
else:
    DynamoDBServiceResource = object

# Module Logger
logger = logging.getLogger(__name__)


class ResourceCreate(_ResourceBase):
    """
    Easily usable class that can be used to create resources in the custom resource code.
    """

    # Sample use case - Non known primary_identifier as input
    # with dynamodb_create(self._request) as DB:
    #     created_resource = self.action()
    #     if created_resource:
    #        DB.set_resource_created(primary_identifier=self._callback_context["working_model"].ReadOnlyIdentifier,
    #                                working_model=self._callback_context["working_model"])

    # Sample use case - Primary identifier known ahead of time
    # with dynamodb_create(self._request, self.primary_identifier) as DB:
    #     #AUTOMATIC CHECKING OF AlreadyEXISTS done by DynamoDB code when we have a primary identifier ahead of
    #     time. IE a user parameter
    #     created_resource = self.action()
    #     if created_resource:
    #        DB.set_resource_created(primary_identifier=self._callback_context["working_model"].ReadOnlyIdentifier,
    #                                working_model=self._callback_context["working_model"])

    def __init__(
        self,
        request: BaseResourceHandlerRequest,
        type_name: str,
        db_resource: DynamoDBServiceResource,
    ):

        super().__init__(
            request=request,
            db_resource=db_resource,
            primary_identifier=None,
            type_name=type_name,
        )

        self._set_resource_created_called = False
        self._current_model: Optional[BaseModel] = None

    def set_resource_created(
        self,
        primary_identifier: str,
        current_model: BaseModel,
    ) -> None:

        if primary_identifier is None:
            raise Exception("Primary Identifier cannot be Null")

        if current_model is None:
            raise Exception("The current_model cannot be Null")

        self._set_resource_created_called = True
        self._primary_identifier = primary_identifier
        self._current_model = current_model

    def _duplicate_primary_identifier(self) -> None:
        if self._db_item_exists():
            raise cloudformation_cli_python_lib.exceptions.AlreadyExists(
                type_name=self._type_name, identifier=self._get_primary_identifier()
            )

    def __enter__(self) -> "ResourceCreate":
        logger.info("DynamoCreate Enter... ")

        # If primary identifier is already set (known with user input) - we need to check if the row already exists
        # Real world example - S3 Bucket Name...

        # NO CANNOT do this - If a primary identifier is known ahead of time and a reinvoke happens -
        # this will fail here

        # We only can call duplicate if we say we created another one
        # In the example real world case - the resource provider must check to
        # see if resource already exists with desired name/primary identifier if possible or fail out appropriately.

        # if self._primary_identifier is not None:
        #     self._duplicate_primary_identifier()

        logger.info("DynamoCreate Enter Complete")
        return self

    def __exit__(
        self,
        exception_type: Optional[Type[BaseException]],
        exception_value: Optional[BaseException],
        traceback: Optional[types.TracebackType],
    ) -> Literal[False]:

        logger.info("DynamoCreate Exit...")

        # End with
        # IF success DB code should create the row if set_resource_created was called
        # If success but set_resource_created was not called - Do nothing in Dynamo -
        #   resource already existed - statemachine execution
        # If error/exception be - no row created and let it propagate upwards

        # Determine if error happened (bad CR to determine if we should delete the item at the end or not.
        # Resource was created...

        if exception_type is None:
            logger.info("Has Failure = False")
            if not self._set_resource_created_called:
                # Resource was already created - nothing to do here - no row needs to be created
                pass
            else:
                # Check if row exists - if it does we need to fail out correctly
                if self._primary_identifier is not None:
                    self._duplicate_primary_identifier()

                logger.info("Row being created")

                self._db_item_insert_without_overwrite(cast(BaseModel, self._current_model))
        else:

            # We failed in creation logic
            logger.info("Has Failure = True, row NOT created")

            # Failed during creation of resource for any number of reasons
            # Assuming it failed with no resource actually created - only valid assumption we can make.
            # Dont create the Row
            pass

        logger.info("DynamoCreate Exit Completed")

        # let exception flourish always
        return False



import logging
import types
from typing import Type, Literal, TYPE_CHECKING, Optional

from cloudformation_cli_python_lib.interface import BaseResourceHandlerRequest

from cf_extension_core.resource_base import ResourceBase

if TYPE_CHECKING:
    from mypy_boto3_dynamodb.service_resource import DynamoDBServiceResource
else:
    DynamoDBServiceResource = object

# Module Logger
logger = logging.getLogger(__name__)


class ResourceDelete(ResourceBase):
    """
    Easily usable class that can be used to Delete resources in the custom resource code.
    """

    # #Delete Handler use case - WE THINK
    # with dynamodb.resource_delete(request=self.request,
    #                               primary_identifier=self._request.previousResourceState.ReadOnlyIdentifier) as DB:
    #
    #     #Arbitrary Code
    #     DB.update_model(model=self._callback_context["working_model"])
    #

    def __init__(
        self,
        request: BaseResourceHandlerRequest,
        db_resource: DynamoDBServiceResource,
        primary_identifier: str,
        type_name: str,
    ):

        super().__init__(
            request=request,
            db_resource=db_resource,
            primary_identifier=primary_identifier,
            type_name=type_name,
        )
        pass

    def __enter__(self) -> "ResourceDelete":
        logger.info("DynamoDelete Enter... ")

        # Check to see if the row/resource is not found
        self._not_found_check()

        logger.info("DynamoDelete Enter Completed")
        return self

    def __exit__(
        self,
        exception_type: Optional[Type[BaseException]],
        exception_value: Optional[BaseException],
        traceback: Optional[types.TracebackType],
    ) -> Literal[False]:

        logger.info("DynamoDelete Exit...")

        if exception_type is None:
            logger.info("Has Failure = False")

            self._db_item_delete()
        else:
            # We failed in delete logic
            logger.info("Has Failure = True, row will not be deleted")

            # Failed during delete of resource for any number of reasons
            # Assuming it failed with a dependency.  Nothing deleted from dynamo DB perspective
            # Dont update the Row

        logger.info("DynamoDelete Exit Completed")

        # let exception flourish always
        return False



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['cf_extension_core']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.26.50,<2.0.0', 'cloudformation-cli-python-lib>=2.1.14,<3.0.0']

setup_kwargs = {
    'name': 'cf-extension-core',
    'version': '0.1.dev20230118182612',
    'description': 'Provides common functionality for Custom resources in CloudFormation.',
    'long_description': '# Summary\n- Helper to enable all types of resource types for create/update/read/list operations\n- Heavily inspired to use dynamodb for resource management.  Supports all native create/read/update/list/delete operations for any resource.\n- Dynamic identifier generation to support any resource identifier use case.  Read Only resources or real resource creation.\n\n# Required extra permissions in each handlers permissions:\n- Due to us using dynamodb as a backend, we need extra permissions to store/retrieve state information from dynamo.  These permissions should be added in addition to any other required permissions by each handler.\n\n  - dynamodb:CreateTable\n  - dynamodb:PutItem\n  - dynamodb:DeleteItem\n  - dynamodb:GetItem\n  - dynamodb:UpdateItem\n  - dynamodb:UpdateTable\n  - dynamodb:DescribeTable\n  - dynamodb:Scan\n\n\n# Development\n- Use of poetry\n- ```commandline\ncurl -sSL https://install.python-poetry.org | python3 -\nexport PATH="/Users/nicholascarpenter/.local/bin:$PATH"\npoetry --version\npoetry add boto3\n\npoetry add --group dev  pytest\n\npoetry install --no-root\npoetry build\npoetry config pypi-token.pypi ""\npoetry publish\n```',
    'author': 'Nick Carpenter',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.1,<4.0.0',
}


setup(**setup_kwargs)
